from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from django.utils.timezone import utc

import datetime

from downtime.models import Period

class Command(BaseCommand):
    help = 'Start downtime due to maintenance/deployment/whatever.'

    def handle(self, *args, **options):
        if getattr(settings, 'USE_TZ', False):
            p = Period(enabled=True, start_time=datetime.datetime.utcnow().replace(tzinfo=utc))
        else:
            p = Period(enabled=True, start_time=datetime.datetime.now())
        p.save()
        self.stdout.write('Successfully started downtime')



from django.db import models
from django.utils.encoding import python_2_unicode_compatible


from downtime.managers import PeriodManager


@python_2_unicode_compatible
class Period(models.Model):
    start_time = models.DateTimeField(blank=True, null=True)
    end_time = models.DateTimeField(blank=True, null=True)
    enabled = models.BooleanField(default=False)

    objects = PeriodManager()

    def __str__(self):
        return "Scheduled Downtime: %s to %s" % (self.start_time, self.end_time)



from django.test import TestCase

from downtime.models import Period
from downtime.tests.factories import PeriodFactory

from django.core.management import call_command

class DowntimeCommandsTest(TestCase):

    def setUp(self):
        self.period = PeriodFactory.create()

    def test_inital_active(self):
        self.assertTrue(Period.objects.is_down(), \
            'Period should deployed initially.')

    def test_downtime_end(self):
        call_command('downtime_end')
        deployed_periods = Period.objects.is_deployment()
        self.assertFalse(deployed_periods, \
            'After running downtime_end, the period should not be deployed.\n \
            Returned: %s ' % deployed_periods)

    def test_downtime_start(self):
        call_command('downtime_start')
        deployed_periods = Period.objects.is_deployment()
        self.assertTrue(deployed_periods,\
            'After running downtime_start, the period should be deployed.\n \
            Returned: %s ' % deployed_periods)



import datetime

from django.db.models import Manager
from django.db.models.query import QuerySet
from django.conf import settings
from django.utils.timezone import utc


class PeriodQuerySet(QuerySet):

    def active(self):
        return self.filter(enabled=True)
    
    def is_deployment(self):
        if getattr(settings, 'USE_TZ', False):
            return self.filter(start_time__lte=datetime.datetime.utcnow().replace(tzinfo=utc), 
                               end_time=None)
        else:
            return self.filter(start_time__lte=datetime.datetime.now(), 
                               end_time=None)
        
    def is_down(self):
        if getattr(settings, 'USE_TZ', False):
            return self.filter(start_time__lte=datetime.datetime.utcnow().replace(tzinfo=utc), 
                               end_time__gte=datetime.datetime.utcnow().replace(tzinfo=utc))
        else:
            return self.filter(start_time__lte=datetime.datetime.now(), 
                               end_time__gte=datetime.datetime.now())


class PeriodManager(Manager):
    def get_queryset(self):
        return PeriodQuerySet(self.model, using=self._db)

    def active(self):
        return self.get_queryset().active()
    
    def is_deployment(self):
        return self.get_queryset().active().is_deployment()
        
    def is_down(self):
        return self.get_queryset().active().is_down() or self.get_queryset().active().is_deployment()



import datetime

import factory

from downtime.models import Period


class PeriodFactory(factory.DjangoModelFactory):
    start_time = datetime.datetime.now()
    end_time = datetime.datetime.now() + datetime.timedelta(days=2)
    enabled = True

    class Meta:
        model = Period



from mock import Mock

from django.test import TestCase


from downtime.middleware import DowntimeMiddleware
from downtime.tests.factories import PeriodFactory


class DowntimeMiddlewareTest(TestCase):

    def setUp(self):
        self.dm = DowntimeMiddleware()
        self.request = Mock()
        self.request.session = {}
        self.period = PeriodFactory.create()

    def test_exempt_paths(self):
        self.request.path = '/admin'
        self.assertIsNone(self.dm.process_request(self.request))

    def test_render(self):
        self.request.path = '/'
        response = self.dm.process_request(self.request)
        self.assertEqual(response.status_code, 503)


from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from django.utils.timezone import utc

import datetime

from downtime.models import Period

class Command(BaseCommand):
    help = 'End downtime after finished maintenance/deployment/whatever.'

    def handle(self, *args, **options):
        objects = Period.objects.is_deployment()
        if getattr(settings, 'USE_TZ', False):
            _now = datetime.datetime.utcnow().replace(tzinfo=utc)
        else:
            _now = datetime.datetime.now()
        for obj in objects:
            obj.end_time = _now
            obj.save()

        self.stdout.write('Successfully ended downtime')



import datetime


from django.test import TestCase

from downtime.tests.factories import PeriodFactory
from downtime.models import Period


class DowntimeModelTest(TestCase):

    def setUp(self):
        self.period = PeriodFactory.create()

    def test_active(self):
        self.assertTrue(Period.objects.active().count())

    def test_inactive(self):
        self.period.enabled = False
        self.period.save()

        self.assertFalse(Period.objects.active().count())

    def test_is_down(self):
        self.assertTrue(Period.objects.is_down().count())

    def test_is_not_down(self):
        self.period.start_time = datetime.datetime.now() - datetime.timedelta(days=4)
        self.period.end_time = datetime.datetime.now() - datetime.timedelta(days=1)
        self.period.save()

        self.assertFalse(Period.objects.is_down().count())



